<?php

/**
 * mod_dmenu
 *
 * @version 1.0
 * @author Creative Pulse
 * @copyright Creative Pulse 2011
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @link http://www.creativepulse.eu
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );


if (!class_exists('Mod_DMenu')) {
	
	class Mod_DMenu {

		function instance_id($new = false) {
			static $num = 0;

			if ($new) {
				$num++;
			}

			return $num;
		}

		function json_esc($input, $esc_lt_gt = true) {
			$result = '';
			$input = (string) $input;
			$conv = array( "'" => "'", '"' => '"', '&' => '&', '\\' => '\\', "\n" => 'n', "\r" => 'r', "\t" => 't', "\b" => 'b', "\f" => 'f' );

			if ($esc_lt_gt) {
				$conv = array_merge($conv, array( '<' => 'u003C', '>' => 'u003E' ));
			}

			for ($i = 0, $len = strlen($input); $i < $len; $i++) {
				$result .= isset($conv[$input{$i}]) ? '\\' . $conv[$input[$i]] : $input[$i];
			}

			return $result;
		}

		function load_menu_item($item) {
			$result = array(
				'type' => 'text', // text, link, link_new, link_popup, separator
				'name' => $item->name,
				'link' => '',
				'href' => '',
				'image' => '',
				'selection' => 0, // 0 = none, 1 = parent, 2 = current
			);

			// Menu Link is a special type that is a link to another item
			if ($item->type == 'menulink') {
				$target_id = 0;

				if (!empty($item->params)) {
					$source_params = new JParameter($item->params);
					$target_id = $source_params->get('menu_item', 0);
				}

				if ($target_id == 0 && preg_match('/Itemid=([0-9]+)/', $item->link, $m))
					$target_id = intval($m[1]);

				if ($target_id > 0) {
					$menu = &JSite::getMenu();
					if ($tmp = $menu->getItem($target_id))
						$item = $tmp;
				}
			}

			static $path = false;
			if ($path === false) {
				$menu = &JSite::getMenu();
				$active = $menu->getActive();
				$path = isset($active) ? array_reverse($active->tree) : null;
			}

			if (!is_array($path) || empty($path))
				{ } // ignore cases
			else if ($path[0] == $item->id)
				$result['selection'] = 2;
			else if (in_array($item->id, $path))
				$result['selection'] = 1;

			$iParams = new JParameter($item->params);
			if ($iParams->get('menu_image') && $iParams->get('menu_image') != -1)
				$result['image'] = '<img src="'.JURI::base(true).'/images/stories/'.$iParams->get('menu_image').'" alt="" border="0" />';

			switch ($item->type) {
				case 'separator' :
					$result['type'] = 'separator';
					break;

				case 'url' :
					if ((strpos($item->link, 'index.php?') !== false) && (strpos($item->link, 'Itemid=') === false)) {
						$item->url = $item->link.'&amp;Itemid='.$item->id;
					} else {
						$item->url = $item->link;
					}
					break;

				default :
					$router = JSite::getRouter();
					$item->url = $router->getMode() == JROUTER_MODE_SEF ? 'index.php?Itemid='.$item->id : $item->link.'&Itemid='.$item->id;
					break;
			}

			// Print a link if it exists
			if ($item->type != 'separator' && $item->url != null)
			{
				// Handle SSL links
				$iSecure = $iParams->def('secure', 0);
				if ($item->home == 1) {
					$item->url = JURI::base();
				} elseif (strcasecmp(substr($item->url, 0, 4), 'http') && (strpos($item->link, 'index.php?') !== false)) {
					$item->url = JRoute::_($item->url, true, $iSecure);
				} else {
					$item->url = str_replace('&', '&amp;', $item->url);
				}

				$result['link'] = $item->url;

				switch ($item->browserNav) {
					case 1:
						// _blank
						$result['href'] = 'href="'.$item->url.'" target="_blank"';
						$result['type'] = 'link_new';
						break;

					case 2:
						// window.open
						$attribs = 'toolbar=no,location=no,status=no,menubar=no,scrollbars=yes,resizable=yes';
						$link = str_replace('index.php', 'index2.php', $item->url);
						$result['href'] = 'href="'.$link.'" onclick="window.open(this.href,\'targetWindow\',\''.$attribs.'\');return false;"';
						$result['type'] = 'link_popup';
						$result['link'] = $link;
						break;

					case 0:
					default:
						// _top
						$result['href'] = 'href="'.$item->url.'"';
						$result['type'] = 'link';
						break;
				}

				if ($result['type'] == 'text')
					$result['type'] = 'link';
			}

			return $result;
		}

		function prepare(&$params) {
			$this->loaded = false;
			$this->menu_name = $params->get('menu_name', '');
			if (empty($this->menu_name)) {
				return;
			}

			$this->loaded = true;

			$db =& JFactory::getDBO();

			$this->iname = 'dmenu' . $this->instance_id(true);


			// setup general parameters
			$this->js_params = array();
			$this->js_params[] = 'iname: "' . $this->iname . '"';
			$this->js_params[] = 'deck_id: "' . $this->iname . '_deck"';
			$this->js_params[] = 'show_container_id: "' . $this->iname . '_show_container"';
			$this->js_params[] = 'show_slides_id: "' . $this->iname . '_show_slides"';
			$this->js_params[] = 'show_area_id: "' . $this->iname . '_show_area"';
			$this->js_params[] = 'show_new_id: "' . $this->iname . '_show_new"';
			$this->js_params[] = 'primary_tpl_id: "' . $this->iname . '_tpl_primary"';
			$this->js_params[] = 'secondary_tpl_id: "' . $this->iname . '_tpl_secondary"';
			$this->js_params[] = 'stock_id: "' . $this->iname . '_stock"';
			$this->js_params[] = 'ani_open_interval: ' . intval($params->get('ani_open_interval', 20));
			$this->js_params[] = 'ani_open_speed: ' . intval($params->get('ani_open_speed', 10));
			$this->js_params[] = 'ani_close_interval: ' . intval($params->get('ani_close_interval', 20));
			$this->js_params[] = 'ani_close_speed: ' . intval($params->get('ani_close_speed', 10));
			$this->js_params[] = 'ani_slide_interval: ' . intval($params->get('ani_slide_interval', 20));
			$this->js_params[] = 'ani_slide_speed: ' . intval($params->get('ani_slide_speed', 10));
			$this->js_params[] = 'ani_fit_interval: ' . intval($params->get('ani_fit_interval', 20));
			$this->js_params[] = 'ani_fit_speed: ' . intval($params->get('ani_fit_speed', 10));
			$this->js_params[] = 'close_timeout: ' . intval($params->get('close_timeout', 500));


			// load regular menu data
			$user =& JFactory::getUser();
			$user_access = (int) $user->get('aid', 0);

			$contentConfig = &JComponentHelper::getParams( 'com_content' );
			$shownoauth = $contentConfig->get('shownoauth');

			$dmenu_component = file_exists(JPATH_SITE . '/administrator/components/com_dmenu/');
			$db->setQuery( sprintf(
				"SELECT m.* %s FROM #__menu m %s WHERE m.menutype = " . $db->Quote($this->menu_name) . " AND m.published='1' %s ORDER BY m.ordering",
				$dmenu_component ? ", d.content" : "",
				$dmenu_component ? "LEFT JOIN #__dmenu_content AS d ON d.menu_id = m.id" : "",
				$shownoauth ? "" : "AND m.access <= $user_access"
			) );
			$this->items = $db->loadObjectList('id');

			$this->menu_base = array();
			if (!empty($this->items)) {
				foreach ($this->items as $k => $mitem) {
					$mi = $this->load_menu_item($mitem);
					$this->items[$k]->in_breadcrumbs = $mi['selection'] > 0;
					$this->items[$k]->selection = $mi['selection'];
					$this->items[$k]->type = $mi['type'];
					$this->items[$k]->link = $mi['link'];
					$this->items[$k]->href = $mi['href'];
					$this->items[$k]->name = $mi['name'];
					$this->items[$k]->image = $mi['image'];
					$this->items[$k]->sub_menu_items = array();
				}
			}
			
			
			// set hierarchy
			foreach ($this->items as $mitem) {
				if ($mitem->parent == '0' && $mitem->menutype == $this->menu_name) {
					$this->menu_base[] = $mitem->id;
				}
				else if (isset($this->items[$mitem->parent])) {
					$this->items[$mitem->parent]->sub_menu_items[] = $mitem->id;
				}
			}
			
			
			// lookup head elements with content
			$this->hdcnt_ids = array();
			$js_data = array();
			foreach ($this->items as $mitem) {
				$is_head = in_array($mitem->id, $this->menu_base);
				$link = substr($mitem->link, 0, 2) == '::' ? $this->json_esc($mitem->link) : '';
				$cnt = false;

				if ($is_head && ($link != '' || !empty($mitem->content) || !empty($mitem->sub_menu_items))) {
					$this->hdcnt_ids[] = $mitem->id;
					$cnt = true;
				}

				$js_data[] = 'card_' . $mitem->id . ': {id: ' . $mitem->id . ', name: "' . $this->json_esc($mitem->name) . '", hd: ' . ($is_head ? 'true' : 'false') . ', cnt: ' . ($cnt ? 'true' : 'false') . ', link: "' . $link . '"}';
			}
			$this->js_params[] = "card_data: {\n			" . implode(",\n			", $js_data) . "\n		}";
			

			// prepare general javascript parameters
			$document =& JFactory::getDocument();
			if ($this->instance_id(false) == 1) {
				$document->addScript('modules/mod_dmenu/js/dmenu.js');
				$document->addScriptDeclaration('document.mod_dmenu_data = [];');
			}

			$document->addScriptDeclaration('
document.mod_dmenu_data.push({
	params: {
		' . implode(",\n		", $this->js_params) . '
	}
});
'
			);
		}
		
		function get_html_content() {
			$data = array();
			$data[$this->menu_name] = array();
			foreach ($this->items as $mitem) {
				if ($mitem->menutype == $this->menu_name) {
					$data[$this->menu_name][] = $mitem->id;
				}
			}
			
			$result = '';
			foreach ($data as $menutype => $data2) {
				foreach ($data2 as $id) {
					$txt = '';
					if (!empty($this->items[$id]->content)) {
						// load custom content
						$txt = $this->items[$id]->content;
					}
					else {
						// load regular menu
						$first = true;
						foreach ($this->items as $mitem) {
							if ($mitem->parent == $id) {
								$str = '';
								if ($mitem->type == 'separator') {
									$str = '<div class="dmenu_dd_item_separator"></div>' . "\n";
									$first = true;
								}
								else if (!empty($mitem->sub_menu_items)) {
									$str = '<div class="dmenu_dd_item_sub"><a href="::submenu::id=' . $mitem->id . '">' . htmlspecialchars($mitem->name) . '</a></div>' . "\n";
								}
								else {
									$str = '<div class="dmenu_dd_item_plain"><a ' . $mitem->href . '>' . htmlspecialchars($mitem->name) . '</a></div>' . "\n";
								}
								
								if ($first) {
									$first = false;
								}
								else {
									$txt .= '<div class="dmenu_dd_item_splitter"></div>' . "\n";
								}
						
								$txt .= $str;
							}
						}
					}

					if ($txt != '') {
						$result .= '<div id="' . $this->iname . '_cnt_' . $id . '">' . $txt . '</div>' . "\n";
					}
				}
			}
			
			return $result;
		}
	}
}

?>